@@ -40,9 +40,6 @@ public final class JITHelpers {
 
 	private static final JITHelpers helpers;
 	private static final Unsafe unsafe;
-	// If JITHelpers class is loaded at startup, so is DecimalFormatHelper. The latter is needed
-	// to optimize DecimalFormat.format(bd.doubleValue()) optimization in jit.
-	private static final DecimalFormatHelper dummyDecFormat = new DecimalFormatHelper();
 
 	private JITHelpers() {
 	}
@@ -545,7 +542,7 @@ public final class JITHelpers {
 	 * at the specified offset and moves towards the end of source array.
 	 * 
 	 * @param s1Value
-	 * 		 	the source character array to serach in
+	 * 		 	the source character array to search in
 	 * @param s1len
 	 * 		    the length of the source array
 	 * @param s2Value
@@ -592,7 +589,7 @@ public final class JITHelpers {
 	 * at the specified offset and moves towards the end of source array.
 	 * 
 	 * @param s1Value
-	 * 		 	the source character array to serach in
+	 * 		 	the source character array to search in
 	 * @param s1len
 	 * 		    the length of the source array
 	 * @param s2Value
