@@ -1,5 +1,3 @@
-package java.lang;
-
 /*******************************************************************************
  * Copyright (c) 1998, 2019 IBM Corp. and others
  *
@@ -21,6 +19,7 @@ package java.lang;
  *
  * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
  *******************************************************************************/
+package java.lang;
 
 import java.io.*;
 import java.util.Map;
@@ -38,6 +37,8 @@ import jdk.internal.reflect.Reflection;
 import jdk.internal.reflect.CallerSensitive;
 import java.util.*;
 import java.util.function.*;
+import com.ibm.gpu.spi.GPUAssist;
+import com.ibm.gpu.spi.GPUAssistHolder;
 
 /**
  * Class System provides a standard place for programs
@@ -161,7 +162,42 @@ static void completeInitialization() {
 	setIn(new BufferedInputStream(new FileInputStream(FileDescriptor.in)));
 		
 	Terminator.setup();
-	
+
+	initGPUAssist();
+
+}
+
+private static void initGPUAssist() {
+	Properties props = internalGetProperties();
+
+	if ((props.getProperty("com.ibm.gpu.enable") == null) //$NON-NLS-1$
+	&& (props.getProperty("com.ibm.gpu.enforce") == null) //$NON-NLS-1$
+	) {
+		/*
+		 * The CUDA implementation of GPUAssist is not enabled by default:
+		 * one of the above properties must be set.
+		 */
+		return;
+	}
+
+	PrivilegedAction<GPUAssist> finder = new PrivilegedAction<GPUAssist>() {
+		@Override
+		public GPUAssist run() {
+			ServiceLoader<GPUAssist.Provider> loaded = ServiceLoader.load(GPUAssist.Provider.class);
+
+			for (GPUAssist.Provider provider : loaded) {
+				GPUAssist assist = provider.getGPUAssist();
+
+				if (assist != null) {
+					return assist;
+				}
+			}
+
+			return GPUAssist.NONE;
+		}
+	};
+
+	GPUAssistHolder.instance = AccessController.doPrivileged(finder);
 }
 
 /**
@@ -293,8 +329,6 @@ private static void ensureProperties(boolean isInitialization) {
 	propertiesInitialized = true;
 
 	java.lang.VersionProps.init(initializedProperties);
-	/* VersionProps.init(systemProperties) above sets java.specification.version value which is used to set java.vm.specification.version. */
-	initializedProperties.put("java.vm.specification.version", initializedProperties.get("java.specification.version")); //$NON-NLS-1$ //$NON-NLS-2$
 
 	String javaRuntimeVersion = initializedProperties.get("java.runtime.version"); //$NON-NLS-1$
 	if (null != javaRuntimeVersion) {
@@ -318,7 +352,7 @@ private static void ensureProperties(boolean isInitialization) {
 
 /* Converts a Map<String, String> to a properties object.
  * 
- * The sytem properties will be initialized as a Map<String, String> type to be compatible
+ * The system properties will be initialized as a Map<String, String> type to be compatible
  * with jdk.internal.misc.VM and java.lang.VersionProps APIs.
  */
 private static void initializeSystemProperties(Map<String, String> mapProperties) {
@@ -1159,12 +1193,34 @@ public interface Logger {
 	 * System loggers levels
 	 */
 	public enum Level {
+		/**
+		 * Enable all logging level messages
+		 */
 		ALL(Integer.MIN_VALUE),
+		/**
+		 * Enable TRACE level messages
+		 */
 		TRACE(400),
+		/**
+		 * Enable DEBUG level messages
+		 */
 		DEBUG(500),
+		/**
+		 * Enable INFO level messages
+		 */
 		INFO(800),
+		/**
+		 * Enable WARNING level messages
+		 */
 		WARNING(900),
+		/**
+		 * Enable ERROR level messages
+		 */
 		ERROR(1000),
+		/**
+		 *
+		 * Disable logging 
+		 */
 		OFF(Integer.MAX_VALUE);
 		
 		final int severity;
